-- UnsheathMove by Vohl --
-- Changelog:
-- V1.0 - First release
-- V1.1 - Added options screen and saved variables for default behaviour
-- V1.2 - Added new event for toggle autorun
-- V1.3 - Added New option and saved variable to enable whilst swimming
-- V1.4 - Added a Debug mode 
--      - Move forward and autorun triggers now only fire when NOT mounted 
--      - Toggle from keybind now only functions when NOT mounted  
-- v1.5 - Added an experimental option to keep weapons sheathed
--      - Added a new timer mechanism with option to enable or disable it!
--      - Cleaned up the code executed when moving forwards or autorun

-- initialise the defaults
local frame = CreateFrame("FRAME", "UnsheathMoveFrame");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
local function eventHandler(self, event, ...)
	if UNSHEATHMOVE_DEFAULT == true then
		UnsheathMoveToggleOnOff = 'Enabled'
		else UnsheathMoveToggleOnOff = 'Disabled'
	end
	if UNSHEATHMOVE_SWIMMING == true then
		UnsheathSwimming = 'Enabled'
	else UnsheathSwimming = 'Disabled'	
	end
	if UNSHEATHMOVE_STAYSHEATHED == true then
		UnsheathStaysheathed = 'Enabled'
		UnsheathStaysheathedToggle = 'Enabled'
		else 
		UnsheathStaysheathed = 'Disabled'
		UnsheathStaysheathedToggle = 'Disabled'
	end
	-- Initialise option to enable/disable timer.. I'll add an option to adjust this in a future release... For now it defaults to 3 seconds
	if UNSHEATHMOVE_TIMER == true then
		UnsheathMoveTimer = 'Enabled'
		else UnsheathMoveTimer = 'Disabled'
	end
end	
frame:SetScript("OnEvent", eventHandler);

UnsheathMoveDebug = 'Disabled'
UnsheathMoveTimerValue = '3'

-- set up key bindings
BINDING_HEADER_UNSHEATHMOVE = "UnsheathMove"
BINDING_NAME_UNSHEATHMOVE = "Toggle"

-- Slash Commands
SLASH_UNSHEATHMOVE1 = '/um';
local function handler(param, editbox)
	if param == 'toggle' then
	UnsheathMoveToggle()
  elseif param == 'config' then
	InterfaceOptionsFrame_OpenToCategory("UnsheathMove")
	InterfaceOptionsFrame_OpenToCategory("UnsheathMove") -- sometimes the blizz options panel doesnt go to the right page.. this forces it to go to the right page
  elseif param == 'debug' then
		if UnsheathMoveDebug == 'Disabled' then	UnsheathMoveDebug = 'Enabled'
		print('UnsheathMove Debug Mode Enabled')
		else UnsheathMoveDebug = 'Disabled'
		print('UnsheathMove Debug Mode Disabled')
	end	
  else
  print('-----------------------------------------');
  print('UnsheathMove is|cFFFFFF00', UnsheathMoveToggleOnOff, '|r');
  print('Possible options are:');
  print('/um toggle - to toggle on or off');
  print('/um config - to open the options screen');
  print('-----------------------------------------');
end
 end
SlashCmdList["UNSHEATHMOVE"] = handler;

--set up options screen
local UnsheathMoveDefaultText = "Enable the addon on startup"
local UnsheathMoveTimerText = "Enable automatic cheking (Uses a timer to check every 3 seconds)"
local UnsheathMoveSwimmingText = "Enable when swimming"
local UnsheathMoveStaysheathedText = "Inverse logic - Keep weapons sheathed"
local unsheathmove = CreateFrame("Frame", "unsheathmoveoptions", InterfaceOptionsFramePanelContainer)
unsheathmove:Hide()
unsheathmove.name = "UnsheathMove"
local title = unsheathmove:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText("UnsheathMove") 
InterfaceOptions_AddCategory(unsheathmove)

--Default Button
local btnUnsheathMoveDefault = CreateFrame("CheckButton", "UnsheathMoveDefaultButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveDefault:SetPoint("TOPLEFT", 16, -35)
btnUnsheathMoveDefault:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_DEFAULT = true
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_DEFAULT = nil
	end
end)
btnUnsheathMoveDefault:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_DEFAULT)
end)
local btnUnsheathMoveDefaultText = btnUnsheathMoveDefault:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveDefaultText:SetPoint("LEFT", btnUnsheathMoveDefault, "RIGHT", 0, 1)
btnUnsheathMoveDefaultText:SetText(UnsheathMoveDefaultText)

--Timer Button
local btnUnsheathMoveTimer = CreateFrame("CheckButton", "UnsheathMoveTimerButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveTimer:SetPoint("TOPLEFT", 16, -65)
btnUnsheathMoveTimer:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_TIMER = true
		UnsheathMoveTimer = 'Enabled'
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_TIMER = nil
		UnsheathMoveTimer = 'Disabled'
	end
end)
btnUnsheathMoveTimer:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_TIMER)
end)
local btnUnsheathMoveTimerText = btnUnsheathMoveTimer:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveTimerText:SetPoint("LEFT", btnUnsheathMoveTimer, "RIGHT", 0, 1)
btnUnsheathMoveTimerText:SetText(UnsheathMoveTimerText)

--Swimming Button
local btnUnsheathMoveSwimming = CreateFrame("CheckButton", "UnsheathMoveSwimmingButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveSwimming:SetPoint("TOPLEFT", 16, -95)
btnUnsheathMoveSwimming:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_SWIMMING = true
		UnsheathSwimming = 'Enabled'
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_SWIMMING = nil
		UnsheathSwimming = 'Disabled'
	end
end)
btnUnsheathMoveSwimming:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_SWIMMING)
end)
local btnUnsheathMoveSwimmingText = btnUnsheathMoveSwimming:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveSwimmingText:SetPoint("LEFT", btnUnsheathMoveSwimming, "RIGHT", 0, 1)
btnUnsheathMoveSwimmingText:SetText(UnsheathMoveSwimmingText)

--Stay sheathed button
local btnUnsheathMoveStaysheathed = CreateFrame("CheckButton", "UnsheathMoveStaysheathedButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveStaysheathed:SetPoint("TOPLEFT", 16, -125)
btnUnsheathMoveStaysheathed:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_STAYSHEATHED = true
		UnsheathStaysheathed = 'Enabled'
		UnsheathStaysheathedToggle = 'Enabled'
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_STAYSHEATHED = nil
		UnsheathStaysheathed = 'Disabled'
		UnsheathStaysheathedToggle = 'Disabled'
	end
end)
btnUnsheathMoveStaysheathed:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_STAYSHEATHED)
end)
local btnUnsheathMoveStaysheathedText = btnUnsheathMoveStaysheathed:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveStaysheathedText:SetPoint("LEFT", btnUnsheathMoveStaysheathed, "RIGHT", 0, 1)
btnUnsheathMoveStaysheathedText:SetText(UnsheathMoveStaysheathedText)


-- Debug mode - This prints the current state to the default chat frame everytime the addon triggers
function UnsheathMoveDebugMode()
	if GetSheathState() == 1 then UnsheathMoveDebugSheathState = 'Sheathed'
	end
	if GetSheathState() == 2 then UnsheathMoveDebugSheathState = 'Unsheathed'
	end
	print('UM:|cFFFFFF00', UnsheathMoveToggleOnOff, '|r:: ', 'SS:|cFFFFFF00', UnsheathStaysheathed, '|r::', 'Status:|cFFFFFF00', UnsheathMoveDebugSheathState, '|r:: ', 'Mounted:|cFFFFFF00', IsMounted(), '|r::')
end

--Debug mode chat frame conditions (stop it from being triigered by the timer)
function UnsheathMoveDebugCheck()
	if UnsheathMoveDebug == 'Enabled' then UnsheathMoveDebugMode()
	end
end

-- Toggle from keybind
function UnsheathMoveToggle()	
	if UnsheathStaysheathed == 'Disabled'
	and not IsMounted() then 
		ToggleSheath()
		UnsheathMoveToggleDefault()
	elseif UnsheathStaysheathed == 'Enabled' then
		ToggleSheath()
		UnsheathMoveStayToggle()
	end
	UnsheathMoveDebugCheck()
end


-- Toggles default and checks against actual sheathstate to set variable
function UnsheathMoveToggleDefault()
		if UnsheathMoveToggleOnOff == 'Disabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Enabled'
		elseif UnsheathMoveToggleOnOff == 'Enabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Disabled'
		else
			UnsheathMoveToggleOnOff = 'Disabled'
		end
end

--Toggles if Stay sheathed is enabled
function UnsheathMoveStayToggle()
		if UnsheathStaysheathedToggle == 'Enabled' and GetSheathState() == 1
		then UnsheathStaysheathedToggle = 'Disabled'
		elseif UnsheathStaysheathedToggle == 'Disabled' and GetSheathState() == 1
		then UnsheathStaysheathedToggle = 'Enabled'
		else UnsheathStaysheathedToggle = 'Enabled'
		end
end

--Function to keep weapons unsheathed
function UnsheathUnsheathFunc()
	if GetSheathState() == 1 
	and (not IsSwimming() or (UnsheathSwimming == 'Enabled' and IsSwimming()))
	and not IsMounted()
	and UnsheathMoveToggleOnOff == 'Enabled'
	and UnsheathStaysheathed == 'Disabled'
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
end

--Function to keep weapons sheathed
function UnsheathStaysheathedFunc()
	if GetSheathState() == 2 
	and UnsheathStaysheathed == 'Enabled'
	and UnsheathStaysheathedToggle == 'Enabled'
	and UnitAffectingCombat("Player") == false
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
end

--unsheath if enabled on moving forward
hooksecurefunc("MoveForwardStart", function()
	UnsheathUnsheathFunc()
	UnsheathStaysheathedFunc()
	UnsheathMoveDebugCheck()
end)

--unsheath if enabled and autorun is toggled
hooksecurefunc("ToggleAutoRun", function()
	UnsheathUnsheathFunc()
	UnsheathStaysheathedFunc()
	UnsheathMoveDebugCheck()
end)

--Timer
local UnsheathTimer = 0 
local function onUpdate(self,elapsed)
	if UnsheathMoveTimer == 'Enabled' then
		UnsheathTimer = UnsheathTimer + elapsed
			if UnsheathTimer >= (tonumber(UnsheathMoveTimerValue)) then
			UnsheathUnsheathFunc()
			UnsheathStaysheathedFunc()
			UnsheathTimer = 0
		end
	end
end
 
local f = CreateFrame("frame")
f:SetScript("OnUpdate", onUpdate)